
from zope.component.interfaces import ObjectEvent
from interfaces import IVersionedFieldModifiedEvent
from zope.interface import implements
from zope.event import notify

class VersionedFieldModifiedEvent(ObjectEvent):
    """A field has been modified, versioning needed"""

    implements(IVersionedFieldModifiedEvent)

    def __init__(self, ob, *fieldnames) :
        """
        Init with a list of (AT) field names.

        >>> from zope.interface import Interface, implements
        >>> class Sample(object):
        ...     implements(Interface)

        >>> obj = Sample()
        >>> obj.field = 42
        >>> notify(VersionedFieldModifiedEvent(obj, "field"))

        """
        super(VersionedFieldModifiedEvent, self).__init__(ob)
        self.fieldnames = fieldnames

def fieldsModified(ob, *fieldnames):
    notify(VersionedFieldModifiedEvent(ob, *fieldnames))





# -*- coding: utf-8 -*-
# Copyright (c) 2006
# Authors:
#   Jean-Paul Ladage <j.ladage@zestsoftware.nl>, jladage
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
#from zope.component import getMultiAdapter
#from zope.viewlet.interfaces import IViewletManager

from plone.app.kss.plonekssview import PloneKSSView
from plone.app.kss.interfaces import IPloneKSSView
from plone.app.kss.interfaces import IPortalObject

from plone.locking.interfaces import ILockable

from zope.interface import implements
from zope import lifecycleevent, event
from zope.publisher.interfaces.browser import IBrowserView

from Acquisition import aq_inner
from Products.Archetypes.event import ObjectEditedEvent
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ZopeTwoPageTemplateFile
from Products.CMFCore.utils import getToolByName

from zope.deprecation import deprecated

import events
from utils import get_econtext

missing_uid_deprecation = \
"This view does not provide a KSS instance UID as required. Falling back to "
"the global context on inline-editing will be removed in Plone 3.5. Please "
"update your templates."

class FieldsView(PloneKSSView):
    
    implements(IPloneKSSView)

    ## KSS methods
   
    view_field_wrapper = ZopeTwoPageTemplateFile('browser/view_field_wrapper.pt')
    edit_field_wrapper = ZopeTwoPageTemplateFile('browser/edit_field_wrapper.pt')

    def renderViewField(self, fieldname, templateId, macro, uid=None):
        """
        renders the macro coming from the view template
        """

        context = self._getFieldContext(uid)
        template = self.getTemplate(templateId, context)

        viewMacro = template.macros[macro]
        res = self.view_field_wrapper(viewMacro=viewMacro,
                                      context=context,
                                      templateId=templateId,
                                      fieldName=fieldname)
        return res

    def getTemplate(self, templateId, context=None):
        """
        traverse/search template
        """

        if not context:
            context = self.context

        template = context.restrictedTraverse(templateId)
        
        if IBrowserView.providedBy(template):
            view = template
            for attr in ('index', 'template', '__call__'):
                template = getattr(view, attr, None)
                if template is not None and hasattr(template, 'macros'):
                    break
            if template is None:
                raise KeyError("Unable to find template for view %s" % templateId)
        return template


    def renderEditField(self, fieldname, templateId, macro, uid=None):
        """
        renders the edit widget inside the macro coming from the view template
        """

        context = self._getFieldContext(uid)
        template = self.getTemplate(templateId, context)
        containingMacro = template.macros[macro]
        fieldname = fieldname.split('archetypes-fieldname-')[-1]
        field = context.getField(fieldname)
        widget = field.widget
        widgetMacro = widget('edit', context)
        
        res = self.edit_field_wrapper(containingMacro=containingMacro,
                                      widgetMacro=widgetMacro,
                                      field=field, instance=context,
                                      mode='edit',
                                      templateId=templateId)

        return res


    # XXX XXX TODO for 3.0.1: see if we really need edit at all or we can remoce it (ree)

    def replaceField(self, fieldname, templateId, macro, uid=None, target=None, edit=False):
        """
        kss commands to replace the field value by the edit widget

        The edit parameter may be used if we are coming from something else
        than an edit view.
        """
        ksscore = self.getCommandSet('core')
        zopecommands = self.getCommandSet('zope')
        plonecommands = self.getCommandSet('plone')

        instance = self._getFieldContext(uid)        

        if edit:
            locking = ILockable(instance, None)
            if locking:
                if not locking.can_safely_unlock():
                    selector = ksscore.getHtmlIdSelector('plone-lock-status')
                    zopecommands.refreshViewlet(selector,
                                                'plone.abovecontent',
                                                'plone.lockinfo')
                    plonecommands.refreshContentMenu()

                    return self.render()
                else: # were are locking the content
                    locking.lock()

        plonecommands.issuePortalMessage('')

        html = self.renderEditField(fieldname, templateId, macro, uid)
        html = html.strip()

        field_id = target or "parent-fieldname-%s" % fieldname
        ksscore.replaceHTML(ksscore.getHtmlIdSelector(field_id), html)
        ksscore.focus("#%s .firstToFocus" % field_id)

        return self.render()

    def replaceWithView(self, fieldname, templateId, macro, uid=None, target=None, edit=False):
        """
        kss commands to replace the edit widget by the field view
        """

        ksscore = self.getCommandSet('core')
        
        instance = self._getFieldContext(uid)        
        locking = ILockable(instance, None)
        if locking and locking.can_safely_unlock():
            locking.unlock()

        html = self.renderViewField(fieldname, templateId, macro, uid)
        html = html.strip()

        field_id = target or "parent-fieldname-%s" % fieldname
        ksscore.replaceHTML(ksscore.getHtmlIdSelector(field_id), html)

        return self.render()

    def saveField(self, fieldname, value=None, templateId=None, macro=None, uid=None, target=None):
        """
        This method saves the current value to the field. and returns the rendered
        view macro.
        """
        # We receive a dict or nothing in value.
        #
        if value is None:
            value = self.request.form.copy()
        instance = self._getFieldContext(uid)        
        field = instance.getField(fieldname)
        value, kwargs = field.widget.process_form(instance, field, value)
        error = field.validate(value, instance, {})
        if not error and field.writeable(instance):
            setField = field.getMutator(instance)
            setField(value, **kwargs)

            # send event that will invoke versioning
            events.fieldsModified(instance, fieldname)

            instance.reindexObject() #XXX: Temp workaround, should be gone in AT 1.5

            descriptor = lifecycleevent.Attributes(IPortalObject, fieldname)
            event.notify(ObjectEditedEvent(instance, descriptor))
            
            return self.replaceWithView(fieldname, templateId, macro, uid, target)
        else:
            if not error:
                # XXX This should not actually happen...
                error = 'Field is not writeable.'
            # Send back the validation error
            self.getCommandSet('atvalidation').issueFieldError(fieldname, error)
            return self.render()

    def _getFieldContext(self, uid):
        if uid is not None:
            rc = getToolByName(aq_inner(self.context), 'reference_catalog')
            return rc.lookupObject(uid)
        else:
            deprecated(FieldsView, missing_uid_deprecation)
            return aq_inner(self.context)
        
class ATDocumentFieldsView(FieldsView):

    def isTableOfContentsEnabled(self):
        getTableContents = getattr(self.context, 'getTableContents', None)
        result = False
        if getTableContents is not None:
            result = getTableContents()
        return result

    def replaceField(self, fieldname, templateId, macro, uid=None, target=None, edit=False):
        if fieldname == "text" and self.isTableOfContentsEnabled():  
            self.getCommandSet('core').setStyle("#document-toc", name="display", value="none")
        FieldsView.replaceField(self, fieldname, templateId, macro, uid=uid, target=target, edit=edit)
        return self.render()

    def replaceWithView(self, fieldname, templateId, macro, uid=None, target=None, edit=False):
        FieldsView.replaceWithView(self, fieldname, templateId, macro, uid=uid, target=target, edit=edit)
        if fieldname == "text" and self.isTableOfContentsEnabled(): 
            self.getCommandSet('core').setStyle("#document-toc", name="display", value="block")
            self.getCommandSet('plone-legacy').createTableOfContents()
        return self.render()
    
    def saveField(self, fieldname, value=None, templateId=None, macro=None, uid=None, target=None):
        FieldsView.saveField(self, fieldname,
                value = value,
                templateId = templateId,
                macro = macro,
                uid = uid,
                target = target,
                )
        if fieldname == "text" and self.isTableOfContentsEnabled(): 
            self.getCommandSet('plone-legacy').createTableOfContents() 
            #manager = getMultiAdapter((self.context, self.request, self),
            #                          IViewletManager,
            #                          name='plone.abovecontentbody')
            #self.getCommandSet('refreshviewlet').refreshViewlet('document-toc',
            #                                                    manager,
            #                                                    'plone.tableofcontents')
        return self.render()

# --
# (Non-ajax) browser view for decorating the field
# --

class ATFieldDecoratorView(BrowserView):

    def getKssUIDClass(self):
        """
        This method generates a class-name from the current context UID.
        """
        uid = aq_inner(self.context).UID()
        
        return "kssattr-atuid-%s" % uid
    
    def _global_kss_inline_editable(self):
        """With a nasty although not unusual hack, we reach
        out to the caller template, and examine the global
        tal variable kss_inline_editable. If it is defined,
        and if it is defined to false, we will prohibit
        inline editing everywhere in the template.
        We apply this 'magic' because the signature to getKssClasses
        is already too complex, and it would be undesirable to
        complicate it some more.
        """
        econtext = get_econtext()
        if econtext is None:
            # tests, probably
            return True
        # kss_inline_editable can be set to false in a template, and this
        # will prohibit inline editing in the page
        kss_inline_editable = econtext.vars.get('kss_inline_editable', True)
        # In addition we also check suppress_preview.
        # suppress_preview is set by CMFEditions, when version preview is shown
        # This means inline editing should be disabled globally
        suppress_preview = econtext.vars.get('suppress_preview', False)
        return kss_inline_editable and not suppress_preview

    def getKssClasses(self, fieldname, templateId=None, macro=None, target=None):
        context = aq_inner(self.context)
        field = context.getField(fieldname)
        # field can be None when widgets are used without fields
        # check whether field is valid
        global_kss_inline_editable = self._global_kss_inline_editable()
        if global_kss_inline_editable and field is not None and field.writeable(context):
            classstring = ' kssattr-atfieldname-%s' % fieldname
            if templateId is not None:
                classstring += ' kssattr-templateId-%s' % templateId
            if macro is not None:
                classstring += ' kssattr-macro-%s' % macro
            if target is not None:
                classstring += ' kssattr-target-%s' % target
            # XXX commented out to avoid macro showing up twice
            # not removed since it might be needed in a use case I forgot about
            # __gotcha
            #else:
            #    classstring += ' kssattr-macro-%s-field-view' % fieldname
        else:
            classstring = ''
        return classstring
    
    def getKssClassesInlineEditable(self, fieldname, templateId, macro=None, target=None):
        classstring = self.getKssClasses(fieldname, templateId, macro, target)
        if classstring:
            classstring += ' inlineEditable'
        return classstring




import inspect

# --
# Getting information about the caller template
# --

class FrameError(Exception):
    pass
    
def get_econtext():
    '''Inspects full context of the caller page

    We access the caller stack and thus the *entire*
    context of the page template, including globals (defines).
    (This could not be passed from the template, normally, due to
    its restricted nature.)

    Some explanation:

    We walk up the the python expression stack until we find the
    Pagetemplates.ZRPythonExpr.__call__ that has econtext
    in the locals. The econtext is a 
    <Products.PageTemplates.Expressions.ZopeContext object that has: 

        vars:           global and local variables combined, readonly.

        setLocal:       set local context variable

        setGlobal:      set global context variable
    
    '''
    frame = inspect.currentframe().f_back
    econtext = None
    try:
        while frame is not None:
            try:
                econtext = frame.f_locals['econtext']
            except KeyError:
                # go one frame up
                frame = frame.f_back
            else:
                # found
                return econtext
        else: 
            # Econtext frame not found
            return None
    finally:
        del frame
        del econtext



# Copyright (c) 2006
# Authors:
#   Jean-Paul Ladage <j.ladage@zestsoftware.nl>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#

import unittest
from Products.PloneTestCase import PloneTestCase
from plone.app.kss.tests.kss_and_plone_layer import KSSAndPloneTestCase
from zope import component
from zope.component import adapter
from kss.core.interfaces import IKSSView
from archetypes.kss.interfaces import IVersionedFieldModifiedEvent
from kss.core.BeautifulSoup import BeautifulSoup

PloneTestCase.setupPloneSite()

@adapter(None, IKSSView, IVersionedFieldModifiedEvent)
def field_modified_handler(ob, view, event):
    ob._eventCaught = True

class FieldsViewTestCase(KSSAndPloneTestCase):

    def afterSetUp(self):
        PloneTestCase.PloneTestCase.afterSetUp(self)
        # commands will be rendered as data structures,
        self.setDebugRequest()
        self.loginAsPortalOwner()
        context = self.portal['front-page']
        # Set up a view 
        self.view = context.restrictedTraverse('saveField')

    # --
    # test the KSS methods
    # --

    def testReplaceField(self):
        self.view.context.changeSkin('Plone Default', self.view.request)
        result = self.view.replaceField('title', 'kss_generic_macros', 'title-field-view')
        self.assertEqual([(r['name'], r['selector'], r['selectorType'])
                             for r in result], [
            ('setStyle', '.portalMessage', 'css'),
            ('replaceInnerHTML', 'kssPortalMessage', 'htmlid'),
            ('setAttribute', 'kssPortalMessage', 'htmlid'),
            ('setStyle', 'kssPortalMessage', 'htmlid'),
            ('replaceHTML', 'parent-fieldname-title', 'htmlid'),
            ('focus', '#parent-fieldname-title .firstToFocus', '')
        ])

    def testReplaceWithView(self):
        self.view.context.changeSkin('Plone Default', self.view.request)
        result = self.view.replaceWithView('title', 'kss_generic_macros', 'title-field-view')
        self.assertEqual([(r['name'], r['selector'], r['selectorType'])
                             for r in result], [
             ('replaceHTML', 'parent-fieldname-title', 'htmlid'),
            ])

    # XXX this test should test without events, so it should stop events listening
    # but we have no more method for that
    def testSaveField(self):
        view = self.view
        result = view.saveField('title', {'title':'My Title'}, 
                                'kss_generic_macros', 'title-field-view')
        self.assertEqual([(r['name'], r['selector'], r['selectorType'])
                             for r in result], [
             ('replaceHTML', 'parent-fieldname-title', 'htmlid'),
            ])
        self.assertEqual('My Title', self.portal['front-page'].Title())
        res = view.saveField('description',
                             {'description':'Woot a funky description!'},
                             'kss_generic_macros', 'description-field-view')
        self.assertEqual('Woot a funky description!', self.portal['front-page'].Description())
    

    # XXX note how these tests are wwrong, obviously events are not listened in this setup
    def testSaveFieldWithEvents(self):
        view = self.view
        result = view.saveField('title', {'title':'My Title'}, 
                                'kss_generic_macros', 'title-field-view')
        self.assertEqual('My Title', self.portal['front-page'].Title())
        res = view.saveField('description',
                             {'description':'Woot a funky description!'},
                             'kss_generic_macros', 'description-field-view')
        self.assertEqual('Woot a funky description!', self.portal['front-page'].Description())


    def testSaveFieldWithValueFromRequest(self):
        view = self.view
        view.request.form['title'] = 'My Title'
        view.request.form['description'] = 'Woot a funky description!'
        result = view.saveField('title', None, 
                                'kss_generic_macros', 'title-field-view')
        self.assertEqual('My Title', self.portal['front-page'].Title())
        res = view.saveField('description',
                             None,
                             'kss_generic_macros', 'description-field-view')
        self.assertEqual('Woot a funky description!', 
                         self.portal['front-page'].Description())

    # XXX this test would only run, if events are really listened 
    # (which they are not) 
    def _XXX_testSaveFieldWithVersioning(self):
        view = self.view
        component.provideHandler(field_modified_handler)
        try:
            res = view.saveField('title', {'title':'My Title'}, 
                                    'kss_generic_macros', 'title-field-view')
            self.assert_(getattr(view.context, '_eventCaught', False))
            view.context._eventCaught = False
            res = view.saveField('description',
                                 {'description':'Woot a funky description!'},
                                 'kss_generic_macros', 'description-field-view')
            self.assert_(getattr(view.context, '_eventCaught', False))
        finally:
            sm = component.getSiteManager()
            sm.unregisterHandler(field_modified_handler)

    # XXX these tests with a provided target node id are a bit silly because the feature
    # is not currently in use in any templates -- we provide a target node id of
    # 'parent-fieldname-title' which is the default anyway -- but the tests make sure
    # this api extension work to some degree.
    
    def testReplaceFieldWithProvidedTargetNodeId(self):
        self.view.context.changeSkin('Plone Default', self.view.request)
        target = 'parent-fieldname-title'
        result = self.view.replaceField('title', 'kss_generic_macros', 'title-field-view', target=target)
        self.assertEqual([(r['name'], r['selector'], r['selectorType'])
                             for r in result], [
            ('setStyle', '.portalMessage', 'css'),
            ('replaceInnerHTML', 'kssPortalMessage', 'htmlid'),
            ('setAttribute', 'kssPortalMessage', 'htmlid'),
            ('setStyle', 'kssPortalMessage', 'htmlid'),
            ('replaceHTML', 'parent-fieldname-title', 'htmlid'),
            ('focus', '#parent-fieldname-title .firstToFocus', '')
        ])

    def testReplaceWithViewWithProvidedTargetNodeId(self):
        self.view.context.changeSkin('Plone Default', self.view.request)
        target = 'parent-fieldname-title'
        result = self.view.replaceWithView('title', 'kss_generic_macros', 'title-field-view', target=target)
        self.assertEqual([(r['name'], r['selector'], r['selectorType'])
                             for r in result], [
             ('replaceHTML', 'parent-fieldname-title', 'htmlid'),
            ])

    def testSaveFieldWithProvidedTargetNodeId(self):
        view = self.view
        target = 'parent-fieldname-title'
        result = view.saveField('title', {'title':'My Title'}, 
                                'kss_generic_macros', 'title-field-view', target=target)
        self.assertEqual([(r['name'], r['selector'], r['selectorType'])
                             for r in result], [
             ('replaceHTML', 'parent-fieldname-title', 'htmlid'),
            ])
        self.assertEqual('My Title', self.portal['front-page'].Title())

    def testReplaceFieldWithProvidedContext(self):
        # set the global context to /news
        context = self.portal['news']
        view = context.restrictedTraverse('saveField')
        context.changeSkin('Plone Default', view.request)

        frontpage_uid = self.portal['front-page'].UID()
        result = view.replaceField('title', 'kss_generic_macros', 'title-field-view', uid=frontpage_uid)

        self.assertEqual([(r['name'], r['selector'], r['selectorType'])
                             for r in result], [
            ('setStyle', '.portalMessage', 'css'),
            ('replaceInnerHTML', 'kssPortalMessage', 'htmlid'),
            ('setAttribute', 'kssPortalMessage', 'htmlid'),
            ('setStyle', 'kssPortalMessage', 'htmlid'),
            ('replaceHTML', 'parent-fieldname-title', 'htmlid'),
            ('focus', '#parent-fieldname-title .firstToFocus', '')
        ])

        # make sure we've got the right context:
        replaceHTML = ''.join([r['params'].get('html', '') for r in result])
        self.assertEqual(u"Welcome to Plone" in replaceHTML, True)       

    def testReplaceWithViewWithProvidedContext(self):
        # set the global context to /news
        context = self.portal['news']
        view = context.restrictedTraverse('saveField')
        context.changeSkin('Plone Default', view.request)

        frontpage_uid = self.portal['front-page'].UID()
        result = view.replaceWithView('title', 'kss_generic_macros', 'title-field-view', uid=frontpage_uid)

        self.assertEqual([(r['name'], r['selector'], r['selectorType'])
                             for r in result], [
             ('replaceHTML', 'parent-fieldname-title', 'htmlid'),
            ])

        # make sure we've got the right context:
        replaceHTML = ''.join([r['params'].get('html', '') for r in result])
        self.assertEqual(u"Welcome to Plone" in replaceHTML, True)       

    def testSaveFieldWithProvidedContext(self):
        # set the global context to /news
        context = self.portal['news']
        view = context.restrictedTraverse('saveField')
        context.changeSkin('Plone Default', view.request)

        frontpage_uid = self.portal['front-page'].UID()
        result = view.saveField('title', {'title': 'My Title'},
                                'kss_generic_macros', 'title-field-view', uid=frontpage_uid)

        self.assertEqual('My Title', self.portal['front-page'].Title())

    
    def testMarkerInATField(self):
        # writeable
        view = self.portal['front-page'].restrictedTraverse('kss_field_decorator_view')
        result = view.getKssClasses('title')
        self.assertEqual(result, ' kssattr-atfieldname-title')
        result = view.getKssClasses('title', 'template')
        self.assertEqual(result, ' kssattr-atfieldname-title kssattr-templateId-template')
        result = view.getKssClasses('title', 'template', 'macro')
        self.assertEqual(result, ' kssattr-atfieldname-title kssattr-templateId-template kssattr-macro-macro')
        self.logout()
        result = view.getKssClasses('title')
        # not writeable
        self.assertEqual(result, '')

    def testMarkerInATFieldInlineEditable(self):
        # writeable
        view = self.portal['front-page'].restrictedTraverse('kss_field_decorator_view')
        result = view.getKssClassesInlineEditable('title', 'template')
        self.assertEqual(result, ' kssattr-atfieldname-title kssattr-templateId-template inlineEditable')
        result = view.getKssClassesInlineEditable('title', 'template', 'macro')
        self.assertEqual(result, ' kssattr-atfieldname-title kssattr-templateId-template kssattr-macro-macro inlineEditable')
        self.logout()
        # not writeable
        result = view.getKssClassesInlineEditable('title', 'template')
        self.assertEqual(result, '')

    def testMarkerInNonATField(self):
        # portal root is not an AT object
        view = self.portal.restrictedTraverse('kss_field_decorator_view')
        result = view.getKssClasses('title')
        # not writeable
        self.assertEqual(result, '')

    def testVersionPreviewIsNotInlineEditable(self):
        """If the kss_inline_editable variable is defined to False
        in a page template, all the fields will be globally prohibited 
        to be editable. This works via the getKssClasses method.
        Similarly, is suppress_preview is set to true, inline
        editing is prohibited. This is set from CMFEditions, in the
        versions_history_form.

        In this test we check that the versions history is not
        inline editable at all.
        """
        obj = self.portal['front-page']
        # Make sure we actually have a revision
        pr = self.portal.portal_repository
        pr.save(obj)
        # Render versions history of the front page
        obj.REQUEST.form['version_id'] = '0'
        rendered = obj.versions_history_form()
        soup = BeautifulSoup(rendered)
        # check that inline edit is not active, by looking at title
        tag = soup.find(id='parent-fieldname-title')
        klass = tag['class']
        # just to check that we are looking at the right bit...
        # XXX but this is no more, we can't check it
        #self.assert_('documentFirstHeading' in klass)
        # ... and now see we are really not inline editable:
        self.assert_('inlineEditable' not in klass)
        self.assert_('kssattr-templateId-' not in klass)
        self.assert_('kssattr-macro-' not in klass)

def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(FieldsViewTestCase),
        ))




from zope.component import adapter
from kss.core.interfaces import IKSSView
from interfaces import IVersionedFieldModifiedEvent

from Products.CMFPlone.utils import base_hasattr
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import PloneMessageFactory as _
from Products.CMFEditions.interfaces.IArchivist import ArchivistUnregisteredError
from Products.CMFEditions.interfaces.IModifier import FileTooLargeToVersionError

# --
# As a temporary solution, we start a new version after a field is 
# instant edited. This is done similarly as on the formcontroller hook. 
# This hook, Products/CMFEditions/skins/update_version_on_edit.py, is
# set up from  Products/CMFEditions/VersionPolicies.py.
# --

@adapter(None, IKSSView, IVersionedFieldModifiedEvent)
def versionObjectBecauseFieldChanged(obj, view, event):
    # XXX I believe we don't need to check if CMFEditions has actually
    # set the hook or not?

    pr = getToolByName(view.context, 'portal_repository')
    isVersionable = pr.isVersionable(obj)
    
    changed = False
    if not base_hasattr(obj, 'version_id'):
        changed = True
    else:
        try:
            changed = not pr.isUpToDate(obj, obj.version_id)
        except ArchivistUnregisteredError:
            # XXX: The object is not actually registered, but a
            # version is set, perhaps it was imported, or versioning
            # info was inappropriately destroyed
            changed = True

    fieldnames = event.fieldnames
    txtfieldnames ='"%s"' % ('", "'.join(fieldnames), )
    # XXX I am not sure we actually want to translate _here_, but 
    # at the moment this seems to be the best policy.
    comment_msg = _('Instant edited field(s) ${fields}' , 
                    mapping=dict(fields=txtfieldnames))
    # XXX unicode is not possible here?
    comment = view.context.translate(comment_msg)
    comment = comment.encode('utf')

    if changed and comment is not None and \
           pr.supportsPolicy(obj, 'at_edit_autoversion') and isVersionable:
        try:
            pr.save(obj=obj, comment=comment)
        except FileTooLargeToVersionError:
            commands = view.getCommandSet('plone')
            commands.issuePortalMessage(
                _("Changes Saved. Versioning for this file has been disabled "
                  "because it is too large."),
                msgtype="warn")
